# -------------------------------------------------------------------------------------
# flo::editor::Html
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      HTML Content component. Lets the user adds chunks of HTML to pages
#
# -------------------------------------------------------------------------------------
package flo::editor::Html;
use MKDoc::XML::XHTML::Linkifier;
use MKDoc::Unicode;
use Unicode::Normalize;
use flo::Standard;
use flo::Editor;
use flo::XHTMLFilter;
use lib::util::Command;
use lib::Exception;
use MKDoc::Util::Text2HTML;
use flo::Standard;
use strict;


##
# $self->new;
# -----------
#   Returns a new flo::editor::Html object based on CGI input
##
sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    
    if (defined $cgi)
    {
	if (defined $cgi->param ($param_name . "_add"))
	{
	    $self->{data} = "";
	}
	else
	{
	    $self->{data} = $cgi->param ($param_name);
	}
    }
    else
    {
	$self->{data} = "";
    }
    $self->{class} = 'html';
    return $self;
}


##
# $self->edit;
# ------------
#   Return the edit HTML chunk for that component
##
sub edit
{
    my $self = shift;
    my $cgi  = $self->{cgi};
    
    my $data = $self->{data};
    $data = text2html ($data);
    
    return new flo::Template ( file => $self->template_path,
			       hash => {
				   block_name  => $self->{param_name},
				   block_value => $data,
				   Direction   => current_document->Direction,
				   Align       => current_document->Align,
			       } )->parse;
}


##
# $self->template_path;
# ---------------------
#   Computes the path of the template which has to be used for
#   that component given the current document language
##
sub template_path
{
    my $self = shift;
    
    my $document_table = table ('Document');
    my $document = $document_table->get ( Full_Path => path_info );
    my $lang = $document->{Lang};
    
    return template ('editor/html', $lang);
}


##
# $self->generate_xml;
# --------------------
#   Generates the XML for that component
##
sub generate_xml
{
    my $self = shift;
    my $data = $self->{data};
    $data = flo::XHTMLFilter->purify ($data);
    
    # convert / remove HTML entities that could
    # not be parsable by XML::Dumper
    my %entities = $data =~ /(\&(?:\#([0-9]+)|\w+)\;)/g;
    delete $entities{'&lt;'};
    delete $entities{'&gt;'};
    delete $entities{'&amp;'};
    delete $entities{'&quot;'};
    while (my ($entity, $decimal) = each %entities)
    {
        # get the character code for this character
        if (defined $decimal and $decimal)
        {
            my $val = chr ($decimal);
	    if ($decimal < 256)
	    {
		if ($decimal > 31)
		{
		    my $u_val = Unicode::String::latin1 ($val);
		    $val = $u_val->utf8;
		}
		else
		{
		    $val = '';
		}
	    }
            $data =~ s/\Q$entity\E/$val/g;
        }
        else
        {
            my $decimal = $MKDoc::Util::Text2HTML::HTML_TO_DECIMAL->{$entity};
            if (defined $decimal and $decimal)
            {
                my $val = chr ($decimal);
		if ($decimal < 256)
		{
		    if ($decimal > 31)
		    {
			my $u_val = Unicode::String::latin1 ($val);
			$val = $u_val->utf8;
		    }
		    else
		    {
			$val = '';
		    }
		}
                $data =~ s/\Q$entity\E/$val/g;
            }
            else
            {
                $data =~ s/\Q$entity\E//g;
            }
        }
    }
    return $data;
}


##
# $self->parse_xml;
# -----------------
#   Parses the XML to reconstruct the component
##
sub parse_xml
{
    my $self = shift;
    my $data = shift;
    $self->{data} = $data;
}


##
# $self->html;
# ------------
#   Returns linkified HTML for that component
##
sub html
{
    my $self = shift;
    my $data = "<body>$self->{data}</body>";
    
    # normalizes the data to form C so that hyperlinking
    # works smoothly...
    $data = NFC ($data);
    u_mark (\$data);
    
    my $tree = new MKDoc::XML::XHTML::Linkifier ( $data );
    $tree->trim_spaces;
    
    $tree->linkify (flo::Editor->links);
    my $res = $tree->text_nice;
    $res =~ s/.*\<body\>\n?//;
    $res =~ s/\n?\<\/body\>.*//;
    return $res;
}


sub to_html { return shift->html }


##
# $self->tidy;
# ------------
#   Tidies the HTML, returns TRUE on tidy success, FALSE otherwise
##
sub tidy
{
    my $self = shift;

    # return 1 if this html component is empty
    return 1 unless ($self->{data});
    my $nolinefeed_data = join '', split /\n|\r/, $self->{data};
    return if ($nolinefeed_data =~ /^\s+$/);

    if (general_config->get ("tidy_command"))
    {
	my $ret = 1;
	try
	{
	    my $tidied_data = $self->html_tidy;
	    $self->{data} = $tidied_data;
	}
	catch
	{
	    $ret = 0;
	};
	return $ret;
    }
    else
    {
	return 1;
    }
}


##
# $obj->html_tidy;
# ----------------
#   Return a clean version of the HTML.
##
sub html_tidy
{
    my $self   = shift;
    my $string = shift || $self->{data};

    my $cfg = general_config();
    my $cmd = $cfg->get ('tidy_command');
    my $tmp_dir = $cfg->get ('temp_dir');
    my ($out, $err) = sysexec ($cmd, $tmp_dir, $string);

    if ($? != 0 && $? != 256)
    {
	throw (new lib::Exception ( code => 'CANNOT_TIDY',
				    info => 'Cannot tidy: ' . $err ) );
    }
    
    # strip everything before <BODY> and after </BODY>
    $out =~ s/^.*<BODY[^>]*>\n?//si;
    $out =~ s\</BODY[^>]*>.*</HTML[^>]*>.*$\\si;
    
    # commented out this 'bugfix' as it seems to
    # create a bug itself (backslashes at end of
    # lines being stripped out)
    # Jean-Michel Hiver 2001.10.23
    
    # hackish fix for \\\n bug
    # $out =~ s/\\(?:\n|\r)//gsim;
    return $out;
}


sub Align { return current_document->Align }
sub Direction { return current_document->Direction }


1;




