# -------------------------------------------------------------------------------------
# flo::editor::Image
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      Image content component. Should let the user upload images
#
# -------------------------------------------------------------------------------------
package flo::editor::Image;
use MKDoc::Util::Text2HTML;
use MKDoc::Unicode;
use flo::Standard;
use flo::Editor;
use strict;

use flo::editor::Mixin::compute_name;
use flo::editor::Mixin::normalize_name;
use flo::editor::Mixin::upload_file;
our @ISA = qw /
  flo::editor::Mixin::compute_name
  flo::editor::Mixin::normalize_name
  flo::editor::Mixin::upload_file
    /;

sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    
    # if CGI is defined, then we probably want to do some stuff
    if (defined $cgi)
    {
	# update alt field
	if (defined $cgi->param ($param_name . "_alt"))
	{
	    my $alt = $cgi->param ($param_name . "_alt");
	    u_mark (\$alt);
	    $self->{alt} = $alt;
	}
	else
	{
	    my $empty = "";
	    u_mark (\$empty);
	    $self->{title} = $empty;
	}
	
	# if the upload field is not empty, then the user
	# just uploaded a new image
	if (defined $cgi->param ($param_name . "_upload"))
	{
	    my $new_image = $cgi->param ($param_name);
	    if ($new_image)
	    {
		my $new_image_friendly = $self->normalize_name ($new_image);
		my $dir                = $self->compute_name;
		my $full_dir           = general_config->get ('image_disk_path') . "/" . $dir;
		$class->upload_file ('image_disk_path', $dir, $new_image_friendly, $cgi->param ($param_name));
		
		# attempts to set a sensible 'alt' image title by looking at the CGI field,
		# then the image name, and then defaulting to 'image_title' by default
		# (which should NEVER happen)
		$self->{image} = "$dir/$new_image_friendly";
		$self->{alt} = $cgi->param ($param_name . "_alt") || $cgi->param ($param_name) || 'image_title';
	    }
	}
	else
	{
	    $self->{image} = $cgi->param ($param_name . "_img");
	}
    }
    
    $self->{class} = 'image';
    return $self;
}


##
# Computes the path of the template which has to be used for
# that component given the current document language.
##
sub template_path
{
    my $self = shift;
    
    my $document_table = table ('Document');
    my $document = $document_table->get ( Full_Path => path_info );
    my $lang = $document->{Lang};
    
    return template ('editor/image', $lang);
}


sub edit
{
    my $self = shift;
    my $cgi  = $self->{cgi};

    return new flo::Template ( file => $self->template_path,
			       hash => {
				   block_name  => $self->{param_name},
				   image       => $self->{image},
				   alt         => $self->{alt},
				   image_url   => general_config->get ("image_http_path") . "/" . $self->{image},
				   Direction   => current_document->Direction,
				   Align       => current_document->Align,
			       } )->parse;
}


sub generate_xml
{
    my $self = shift;
    my $data = "";
    if ($self->{image})
    {
	my $src = $self->{image};
	my $alt = text2html ($self->{alt});
	my $chr_11 = quotemeta (chr (11));
	$alt =~ s/\n|\r|$chr_11/ /gsm;
	$alt =~ s/\s+/ /g;
	$data = qq |<img src="$src" alt="$alt" />|;
    }
    
    u_mark (\$data);
    return $data;
}


sub parse_xml
{
    my $self = shift;
    my $line = shift;
    if ($line)
    {
	my ($img, $alt) = $line =~ /<img src=\"(.*?)\" alt=\"(.*)\" \/>/;
	$self->{image}  = $img;
	$self->{alt}    = $alt || $self->{image};
	$self->{alt}    = html2text ($alt);
    }
    else
    {
	$self->{image} = "";
	$self->{alt}   = "";
    }
}


sub html
{
    my $self = shift;
    my $src  = general_config->get ("image_http_path") . "/" . $self->{image};
    return '<img src="' . $src . '" alt="' . text2html ($self->{alt}) . '" />';
}


sub Align { return current_document->Align }
sub Direction { return current_document->Direction }

1;
