# -------------------------------------------------------------------------------------
# flo::editor::Photo
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      Image content component. Should let the user upload images
#
# -------------------------------------------------------------------------------------
package flo::editor::Photo;
use MKDoc::Util::Text2HTML;
use MKDoc::XML::Dumper;
use MKDoc::Unicode;
use flo::Standard;
use Image::Magick;
use flo::Editor;
use strict;
use Carp;

use flo::editor::Mixin::compute_name;
use flo::editor::Mixin::normalize_name;
use flo::editor::Mixin::upload_file;
our @ISA = qw /
  flo::editor::Mixin::compute_name
  flo::editor::Mixin::normalize_name
  flo::editor::Mixin::upload_file
    /;


sub new
{
    my $class = shift;
    $class    = ref $class || $class;
    my $self  = bless { @_ }, $class;
    
    my $cgi = $self->{cgi};
    my $param_name = $self->{param_name};
    
    # if CGI is defined, then we probably want to do some stuff
    if (defined $cgi)
    {
	# update alt field
	if (defined $cgi->param ($param_name . "_alt"))
	{
	    my $alt = $cgi->param ($param_name . "_alt");
	    my $description = $cgi->param ($param_name . "_description");
	    my $creator = $cgi->param ($param_name . "_creator") || $class->_current_editor_name;
	    my ($created_day, $created_month, $created_year) =
		$class->_fix_date ( $cgi->param ($param_name . "_created_day"),
				    $cgi->param ($param_name . "_created_month"),
				    $cgi->param ($param_name . "_created_year") );
	    
	    my $rights = $cgi->param ($param_name . "_rights");
	    my $coverage = $cgi->param ($param_name . "_coverage");
	    u_mark (\$alt);
	    u_mark (\$description);
	    u_mark (\$creator);
	    u_mark (\$created_day);
	    u_mark (\$created_month);
	    u_mark (\$created_year);
	    u_mark (\$rights);
	    u_mark (\$coverage);
	    $self->{alt} = $alt;
	    $self->{description} = $description;
	    $self->{creator} = $creator;
	    $self->{created_day} = $created_day;
	    $self->{created_month} = $created_month;
	    $self->{created_year} = $created_year;
	    $self->{rights} = $rights;
	    $self->{coverage} = $coverage;
	}
	else
	{
	    my $empty = "";
	    u_mark (\$empty);
	    $self->{alt} = $empty;
	    $self->{alt} = $empty;
	    $self->{description} = $empty;
	    $self->{created_day} = $empty;
	    $self->{created_month} = $empty;
	    $self->{created_year} = $empty;
	    $self->{rights} = $empty;
	    $self->{coverage} = $empty;
	    
	    my $creator = $class->_current_editor_name;
	    u_mark (\$creator);
	    $self->{creator} = $creator;
  	}
	
	# if the upload field is not empty, then the user
	# just uploaded a new image
	if (defined $cgi->param ($param_name . "_upload"))
	{
	    my $new_image = $cgi->param ($param_name);
	    if ($new_image)
	    {
		my $new_image_friendly = $self->normalize_name ($new_image);
		
		my $dir           = $self->compute_name;
		my $dir_thumbnail = "$dir/thumbnail";
		my $dir_scaled    = "$dir/scaled";
		
		my $full_dir           = general_config->get ('image_disk_path') . "/" . $dir;
		my $full_dir_thumbnail = general_config->get ('image_disk_path') . "/" . $dir_thumbnail;
		my $full_dir_scaled    = general_config->get ('image_disk_path') . "/" . $dir_scaled;
		
		mkdir $full_dir           or confess "Cannot create $dir";
		mkdir $full_dir_thumbnail or confess "Cannot create $full_dir";
		mkdir $full_dir_scaled    or confess "Cannot create $dir_scaled";
		
		$class->upload_file ('image_disk_path', $dir, $new_image_friendly, $cgi->param ($param_name));
		
		# we uploaded the photo, now we need to make the scaled
		# and thumbnailed versions of the image
		my $config = new lib::Config ('flo.plugin.Photo');
		my $scaled_max_x = $config->get ('scaled_max_x');
		my $scaled_max_y = $config->get ('scaled_max_y');
		my $thumb_max_x  = $config->get ('thumb_max_x');
		my $thumb_max_y  = $config->get ('thumb_max_y');
		
		my $image = new Image::Magick;
		$image->Read ("$full_dir/$new_image_friendly");
		$image->Resize (geometry => $scaled_max_x . 'x' . $scaled_max_y, profile => '*');
		$image->Write ("$full_dir_scaled/$new_image_friendly");
		$image->Resize (geometry => $thumb_max_x  . 'x' . $thumb_max_y,  profile => '*');
		$image->Write ("$full_dir_thumbnail/$new_image_friendly");
		
		# attempts to set a sensible 'alt' image title by looking at the CGI field,
		# then the image name, and then defaulting to 'image_title' by default
		# (which should NEVER happen)
		$self->{image} = "$dir/$new_image_friendly";
		$self->{alt} = $cgi->param ($param_name . "_alt") || $cgi->param ($param_name) || 'image_title';
	    }
	}
	else
	{
	    $self->{image} = $cgi->param ($param_name . "_img");
	}
    }
    
    $self->{class} = 'photo';
    return $self;
}


##
# _fix_date ($day, $month, $year);
# --------------------------------
#   Fix the date if it doesn't make sense
##
sub _fix_date
{
    my $class = shift;
    my $day = shift;
    my $month = shift;
    my $year = shift;
    
    my $localtime = [ localtime ];
    my $current_year = $localtime->[5] + 1900;    
    $year = $current_year unless $year =~ /^[+-]?\d+$/;
    
    $month > 0 and $month < 13 or $month = 1;
    
    $day = 1  if ($day < 1);
    $day = 28 if ( ( ($year % 4) and ($month == 2) ) and ($day > 28) );
    $day = 29 if ( ( ($year %4 == 0) and ($month == 2) ) and ($day > 29) );
    $day = 30 if ( ( ($month == 4) or ($month == 6) or
		     ($month == 9) or ($month == 11) ) and $day > 30 );
    $day = 31 if ( ( ($month == 1) or ($month == 3)  or
		     ($month == 5) or ($month == 7)  or
		     ($month == 8) or ($month == 10) or
		     ($month == 12) ) and $day > 31 );
    
    return ($day, $month, $year);
}


##
# _current_editor_name;
# ---------------------
#   Returns the name of the current editor
##
sub _current_editor_name
{
    my $class = shift;
    my $session = session();
    my $user_id = $session->{'Editor_ID'};
    return 'nobody' unless (defined $user_id and $user_id);
    my $editor_table = table ('Editor');
    my $editor = $editor_table->get ($user_id);
    return 'nobody' unless (defined $editor);
    return "$editor->{First_Name} $editor->{Family_Name}"
	if ( defined $editor->{First_Name} and $editor->{First_Name} and
	     defined $editor->{Family_Name} and $editor->{Family_Name} );
    return $editor->{Family_Name};
}


##
# Computes the path of the template which has to be used for
# that component given the current document language
##
sub template_path
{
    my $self = shift;
    
    my $document_table = table ('Document');
    my $document = $document_table->get ( Full_Path => path_info );
    my $lang = $document->{Lang};
    
    return template ('editor/photo', $lang);
}


sub edit
{
    my $self = shift;
    my $cgi  = $self->{cgi};
    
    my %hash = %{$self};
    
    $hash{block_name} = $self->{param_name};
    $hash{photo_url}  = $self->Thumbnail_URI;
    $hash{Direction}  = current_document->Direction;
    $hash{Align}      = current_document->Align;
    
    return new flo::Template ( file => $self->template_path,
			       hash => \%hash )->parse;
}


sub Thumbnail_URI
{
    my $self = shift;
    my $file = $self->{image};
    my @path = split '/', $file;
    my $filename = pop (@path);
    push @path, 'thumbnail';
    push @path, $filename;
    my $new_file = join '/', @path;
    return general_config->get ("image_http_path") . "/" . $new_file;
}


sub Scaled_URI
{
    my $self = shift;
    my $file = $self->{image};
    my @path = split '/', $file;
    my $filename = pop (@path);
    push @path, 'scaled';
    push @path, $filename;
    my $new_file = join '/', @path;
    return general_config->get ("image_http_path") . "/" . $new_file;
}


sub Original_URI
{
    my $self = shift;
    my $file = $self->{image};
    return general_config->get ("image_http_path") . "/" . $file;
}


sub DC_Identifier
{
    my $self = shift;
    # dangerous
    my $path_info = $self->Original_URI;
    my $cgix = cgi->new;
    $cgix->path_info ($path_info);
    return $cgix->self_url;
}


sub generate_xml
{
    my $self = shift;
    return pl2xml ($self);
}


sub parse_xml
{
    my $self = shift;
    my $class = ref $self;
    my $data = shift;
    if ($data)
    {
	if ($data =~ /<img src/)
	{
	    my ($img, $alt) = $data =~ /<img src=\"(.*?)\" alt=\"(.*)\" \/>/;
	    $self->{image}  = $img;
	    $self->{alt}    = $alt || $self->{image};
	    $self->{alt}    = html2text ($alt);
	}
	else
	{
	    my $hash = xml2pl ($data);
	    for (keys %{$hash}) { $self->{$_} = $hash->{$_} }
	}
    }
    else
    {
	my $empty = "";
	u_mark (\$empty);
	$self->{alt} = $empty;
	$self->{alt} = $empty;
	$self->{description} = $empty;
	$self->{created_day} = $empty;
	$self->{created_month} = $empty;
	$self->{created_year} = $empty;
	$self->{rights} = $empty;
	$self->{coverage} = $empty;
	
	my $creator = $class->_current_editor_name;
	u_mark (\$creator);
	$self->{creator} = $creator;
    }
}


sub html
{
    my $self = shift;
    return '<img src="' . $self->Thumbnail_URI . '" alt="' . text2html ($self->{alt}) . '" />';
}


##
# $self->Scaled_HTML_URI;
# -----------------------
#   Computes the URI to see the results for that poll
##
sub Scaled_HTML_URI
{
    my $self = shift;
    my $cfg = new lib::Config ('flo.plugin.Photo');
    my $http_name = $cfg->get ('http_name');
    
    my $id = $self->{image};
    $id =~ s/\/.*//;
    return $id . '_' . $http_name;
}


sub Align { return current_document->Align }
sub Direction { return current_document->Direction }


1;

